#ifndef ILP_RRRMT_HPP_
#define ILP_RRRMT_HPP_

#include <set>
#include <string>
#include <limits>

#include "io.hpp"
#include "costfoo.hpp"
#include "common.hpp"
#include "helpers.hpp"
#include "rearrangements.hpp"
#include "tdl.hpp"

using namespace std;

class ilp_rrrmt : public ordered{
	private:

	public:
	/**
	 * construct an empty scenario
	 */
	ilp_rrrmt();
	/**
	 * constructor: get the scenario from g1 to g2
	 * @param[in] g1 a genome
	 * @param[in] g2 another genome
	 * @param[in] cst cost function
	 * @param[out] objV stores the object value of rrrmt
	 * @param[in] output=TRUE Gurobi output is shown, output=FLASE no output will we produced
	 */
	//ilp_rrrmt(const genom &g1, const genom &g2, const costfoo *cst, float &objV, bool output);

	/**
	 * constructor: get the scenario from g1 to g2
	 * @param[in] g1 a genome
	 * @param[in] g2 another genome
	 * @param[in] cst costs of the rearrangements
	 * @param[out] objective value will be stored in obj
	 * @param[in/out] ms the score of scenario, including score of children
	 * @param[in/out] mr the rearrangement of scenario, including rrrmts of children
	 * @param[out] optimal: TRUE==ILP gives optimal solution
	 * @param[out] is_cutoff: true iff no solution could be found duw to cutoff violation
	 * @param[out] time_exceeded; true iff solution wasn't found due to time constraint violation
	 * @param[out] alternative_scen: scenario rearranging g1 into g2
	 * @param[in] time_limit: time limit of calculation
	 * @param[in] cdps the scores of the child nodes
	 * @param[in] cdpr the rearrangement scenarios of the children
	 * @param[in] name is the name of the stored model: mane.mps
	 * @param[in] all get all scenarios
	 * @param[in] output: if TRUE GUROBI output is shown, FALSE there is no GUROBI output
	 * @param[in] mapping
	 * @param[in] origin is used to reconstruct
	 * @param[in] useCrex2: True -> ILP using Crex2 is used, otherwise normal ILP
	 * @param[in] use_cutoff: true -> cutoff used for calculation (default: false)
	 * @param[in] cutoff: an upper bound on the minimum solution; used to set a cutoff
	 * @param[in] fixed_numb_rrrmt: true iff ILP only calculates the solution for a given number of rrrmts "exact_distance"
	 * @param[in] exact_distance: if fixed_numb_rrrmt: the exact number of rrrmt that can be used for a reconstruction
	 **/

	ilp_rrrmt(const genom &g1,
			  const genom &g2,
			  const costfoo *cst,
			  double &obj,
			  float &ms,
			  rrrmt* &mr,
			  bool &optimal,
			  bool &is_cutoff,
			  bool &time_exceeded,
			  vector<alternative> &alternative_scen,
			  double time_limit = std::numeric_limits<double>::max(),
			  const vector<vector<float> > &cdps = vector<vector<float> >(),
			  const vector<vector<rrrmt*> > &cdpr = vector<vector<rrrmt*> >(),
			  string name = "",
			  unsigned int all = 1,
			  bool output = false,
			  const vector<vector<int> > &mapping = vector<vector<int> >(),
			  const genom &origin = genom(),
			  bool useCrex2 = false,
			  bool use_cutoff =false,
			  float cutoff = std::numeric_limits<unsigned>::max(),
			  bool fixed_numb_rrrmt = false,
			  unsigned int exact_distance = 0);

	/**
		 * constructor: get the scenario from g1 to id
		 * @param[in] g1 genome that has to be sorted to id
		 * @param[in] cst cost function
		 * @param[out] objV stores the object value of rrrmt
		 */
	ilp_rrrmt(const genom &g1, const costfoo *cst, float &objV);

	/**
	 * copy constructor
	 * @param[in] c the original
	 */
	ilp_rrrmt(const ilp_rrrmt &c );

	/**
	 * destructor
	 */
	~ilp_rrrmt();

	/**
	 * @return "ilp_rrrmt"
	 */
	string typestr() const;
};

#endif/*ILP_RRRMT_HPP_*/
